local rs = require "libs/resolution_solution"

camera = {}

camera.x = 0 
camera.y = 0 
camera.r = 0 
camera.ox = 0 
camera.oy = 0
camera.zoom = 1
camera.smooth = 0.1
camera.layers = {} 

local limit_camera = false
local map_width = 0 
local map_height = 0 
local screen_width = 0 
local screen_height = 0

function camera.draw()
  local bx, by = camera.x, camera.y 
  
  for _, v in ipairs(camera.layers) do 
    camera.x = bx * v.scale 
    camera.y = by * v.scale 
    
    camera.set()
      v.draw()
    camera.unset()
  end
end

function camera:set()
  love.graphics.push()
  love.graphics.translate(screen_width/2,screen_height/2)
  love.graphics.rotate(-camera.r)
  love.graphics.translate(-screen_width/2,-screen_height/2)
  
  love.graphics.translate(-camera.x, -camera.y)
  love.graphics.scale(1/camera.zoom,1/camera.zoom)
end 

function camera:unset()
  love.graphics.pop()
end 

function camera.set_zoom(z)
  camera.zoom = camera.zoom * (z or 1)
end

function camera.rotate(deg)
  camera.r = camera.r + math.rad(deg)
end

function camera.set_position(x,y)
  local x = (x/camera.zoom) - camera.ox 
  local y = (y/camera.zoom) - camera.oy 
  
  camera.x = camera.x + (x - camera.x) * camera.smooth
  camera.y = camera.y + (y - camera.y) * camera.smooth
  
  if limit_camera then
    camera.x = math.max(0, math.min(map_width/camera.zoom - screen_width, camera.x))
    camera.y = math.max(0, math.min(map_height/camera.zoom - screen_height, camera.y))
  end
end

function camera.set_origin(ox, oy)
  camera.ox = ox 
  camera.oy = oy
end

function camera.set_limit(bool, map_w, map_h, screen_w, screen_h)
  limit_camera = bool 
  map_width, map_height = map_w, map_h
  screen_width, screen_height = screen_w, screen_h
end 

local shakeTimer = 0 
local shakeDuration = 1

function camera.shake(dt, intensity_x, intensity_y, smoothness)
  local intensity_y = intensity_y or intensity_x
  local x = math.random(-intensity_x, intensity_x)
  local y = math.random(-intensity_y, intensity_y)
  camera.x = camera.x + x
  camera.y = camera.y + y
end

function camera.new_layer(scale, func)
  table.insert(camera.layers, {draw = func, scale = scale})
  table.sort(camera.layers, function(a,b) return a.scale < b.scale end)
end 

function camera.debug(x,y,r,sx,sy)
  love.graphics.setColor(0.1,0.8,0.1,1)
  love.graphics.print(table.concat({
    "cam_x:"..math.floor(camera.x*camera.zoom),
    "cam_y:"..math.floor(camera.y*camera.zoom),
    "cam_zoom:"..string.format("%.2f",camera.zoom),
    "cam_rotation:"..math.deg(camera.r),
    "cam_smooth:"..camera.smooth,
    "cam_limit:"..tostring(limit_camera),
  },"\n"),x,y,r,sx)
end

return camera
