local gamestates = require "gamestates/gamestates"

local levels = require "level_builder"
local camera = require "camera"

require "classes/player"
local player = nil

local rs = require "libs/resolution_solution"

local playground = {}

function playground.load()
  local level_data = require "levels/parallax_test"
  levels.build_level(level_data)
  
  player = Player.new(100,100)
  
  camera.set_zoom(0.9)
  camera.set_origin(rs.game_width/2, rs.game_height/2)
  camera.set_limit(true, levels.map_width, levels.map_height,rs.game_width, rs.game_height)
end

function playground.update(dt)
  levels.update(dt)
  player:update(dt)
  
  local x, y = player.physics.body:getPosition()
  camera.set_position(x,y)
end

function playground.draw()
  levels.draw({"background1"},-0.006,0)
  levels.draw("layer1")
  player:draw()
  if debug_on > 0 then
    levels.debug()
    player:debug(0,0,0,0,0,true)
  end
  levels.draw("layer2",0.02,-0.02)
end

-- Joystick Callbacks
function playground.joystickpressed(joy, button)
  player:joystickpressed(joy, button)
end

-- Keyboard Callbacks
function playground.keypressed(key)
  player:keypressed(key)
end

-- Physics Callbacks 
function playground.beginContact(a,b,coll)
  player:beginContact(a,b,coll)
end

function playground.endContact(a,b,coll)
  player:endContact(a,b,coll)
end

function playground.preSolve(a,b,coll)
  player:preSolve(a,b,coll)
end

function playground.postSolve(a,b,coll,c,d)
  player:postSolve(a,b,coll,c,d)
end 

function playground.debug()
  if debug_on > 0 then
    player:debug(5,5,0,1.3,1.3,false,true,true)
  
    -- Game screen center
    love.graphics.setColor(1,0,0,1)
    love.graphics.line(rs.game_width/2, 0, rs.game_width/2, rs.game_height)
  
    love.graphics.setColor(0,1,0,1)
    love.graphics.line(0, rs.game_height/2, rs.game_width, rs.game_height/2)
  end
end

function playground.exit()
  
end 

return playground
