camera = require "camera"

local levels = {}
levels.collider = {}
levels.top_left_x = 0
levels.top_left_y = 0
levels.tile_width = 32
levels.tile_height = 32
levels.x_edge = 0 
levels.y_edge = 0
levels.map_width = 0 
levels.map_height = 0
levels.sprite_width = 16
levels.sprite_height = 16
levels.data = {}
levels.tileSprites = {}
levels.layers = 1

function levels.update(dt)
  --[[if not levels.sprite_setup then
    levels.sprite_setup = true
    levels.tileSprites = levels.data.load_sheets()
    levels.set_quads(levels.tileSprites)
  end]]
end

function levels.draw(layers,scale_x,scale_y)
  love.graphics.setColor(1,1,1,1)
  -- Draw two or more layers
  if type(layers) == "table" then
    for _, layer in ipairs(layers) do 
      if scale_x ~= nil then 
        local old_x = camera.x 
        local old_y = camera.y
        local old_zoom = camera.zoom
        
        camera.x = camera.x * scale_x
        camera.y = camera.y * scale_y 
        
        camera.set()
          love.graphics.scale(camera.zoom)
          love.graphics.setColor(0.5,0.5,0.5,1)
          levels.sprite_draw(layer)
          love.graphics.setColor(1,1,1,1)
        camera.unset()
        camera.x = old_x
        camera.y = old_y
        camera.zoom = old_zoom
      else
        levels.sprite_draw(layer)
      end
    end 
    
  -- Draw one layer
  else 
    if scale_x ~= nil then
      local old_x = camera.x 
      local old_y = camera.y
      local old_zoom = camera.zoom
      
      camera.x = camera.x * scale_x
      camera.y = camera.y * scale_y
      
      camera.set()
        love.graphics.scale(camera.zoom)
        love.graphics.setColor(0.5,0.5,0.5,1)
        levels.sprite_draw(layers)
        love.graphics.setColor(1,1,1,1)
      camera.unset()
      camera.x = old_x
      camera.y = old_y
      camera.zoom = old_zoom
    else 
      levels.sprite_draw(layers)
    end
  end
end

function levels.set_sprites(tileSheet_name)
  for index, name in ipairs(tileSheet_name) do 
    local sprite_table = {}
    table.insert(sprite_table, assets[name])
    table.insert(levels.tileSprites, sprite_table)
  end
end

function levels.add_collider(userData,x,y,w,h, friction)
  local collider = {}
  
  collider.body = love.physics.newBody(World, x, y, "static")
  collider.shape = love.physics.newRectangleShape(w/2,h/2, w, h)
  collider.fixture = love.physics.newFixture(collider.body, collider.shape, 1)
  collider.fixture:setUserData(userData)
  collider.fixture:setFriction(friction)

  table.insert(levels.collider, collider)
end

function levels.build_level(level_data)
  levels.data = level_data
  levels.set_sprites(level_data.tileSheets_names)
  levels.set_quads(levels.tileSprites)
  levels.map_width, levels.map_height = levels.get_map_dimensions(level_data)
  
  -- Fake loop for (Danger)
  local row_index = 1
  while row_index <= #level_data.collider do
    local row = level_data.collider[row_index]
    local col_index = 1
    
    while col_index <= #row do 
      local tile = row[col_index]
      local adjacents = 0
      local userData = nil
      local friction = 0
         
      if tile == 1 then
        userData = "ground"
        friction = 0.2
      end
      if tile == 2 then
        userData = "ground"
        friction = 0.2
      end
      if tile == 3 then
        userData = "ground"
        friction = 0.2
      end
      
      if type(userData) == "string" then
        local adjacents = levels.check_adjacents(row_index, col_index, tile, level_data)
        local new_x = levels.top_left_x + (levels.tile_width * (col_index - 1))
        local new_y = levels.top_left_y + (levels.tile_height * (row_index - 1))
        
        levels.add_collider(userData, new_x, new_y, levels.tile_width + (levels.tile_width * adjacents), levels.tile_height, friction)
        col_index = col_index + adjacents
      end
      col_index = col_index + 1
    end
    row_index = row_index + 1
  end
end

function levels.check_adjacents(new_row_index, new_col_index, tile, level_data)
  local adjacents = 0 
  
  for row_index = new_row_index, #level_data.collider do
    local row = level_data.collider[row_index]
    
    for col_index = new_col_index, #row do 
      local next_tile = row[col_index]
  
      if next_tile == tile then
        adjacents = adjacents + 1
        if col_index == #row then
          return adjacents - 1
        end 
        
      else 
        return adjacents - 1
      end
    end
  end
  
  return adjacents
end

function levels.sprite_draw(layer)
 local spriteMap = levels.data[layer]
 
 if type(spriteMap) == "table" then 
    for row_index, row in ipairs(spriteMap) do
      for col_index, tile in ipairs(row) do 
        local spriteX = (levels.tile_width * (col_index - 1)) + levels.x_edge
        local spriteY = (levels.tile_height * (row_index - 1)) + levels.y_edge
       
        if tile == 1 then -- Normal brick
          levels.set_draw(levels.tileSprites[1][1], levels.tileSprites[1][2][1], spriteX, spriteY)
        end
        if tile == 2 then -- Desert brick
          levels.set_draw(levels.tileSprites[1][1], levels.tileSprites[1][2][2], spriteX, spriteY)
        end
        if tile == 3 then -- Ice brick
          levels.set_draw(levels.tileSprites[1][1], levels.tileSprites[1][2][3], spriteX, spriteY)
        end
      end
    end
  end
end

function levels.set_quads(sprite_table)
  for data_index, sprite_data in ipairs(sprite_table) do 
    local rows = sprite_data[1]:getHeight()/levels.sprite_height
    local cols = sprite_data[1]:getWidth()/levels.sprite_width
    local quads = {}
    
    for row_index=1, rows do 
      for col_index=1, cols do 
        local quad_x = (levels.sprite_width * (col_index - 1)) + levels.x_edge
        local quad_y = (levels.sprite_height * (row_index - 1)) + levels.y_edge
        
        table.insert(quads,
          love.graphics.newQuad(quad_x,quad_y,
            levels.sprite_width, levels.sprite_height,
            sprite_data[1]:getWidth(), sprite_data[1]:getHeight()
            ))
      end
    end
    
    levels.tileSprites[data_index][2] = quads
  end
end 

function levels.set_draw(sheet,quad,x,y)
  
  love.graphics.draw(sheet,quad,x,y,0,2,2)
end

function levels.get_map_dimensions(level_data, layer)
  local row_len = 0
  local layer = layer or "layer1"
  
  for row_index, row in ipairs(level_data[layer]) do 
    row_len = math.max(row_len, #row)
  end
  
  local map_width = row_len * levels.tile_width
  local map_height = #level_data[layer] * levels.tile_height
  
  return map_width, map_height
end

function levels.debug()
  -- Collisions
  love.graphics.setColor(0,1,0,1)
  for _, obj in ipairs(levels.collider) do
    love.graphics.polygon("line",
      obj.body:getWorldPoints(
        obj.shape:getPoints()))
  end
  love.graphics.setColor(1,1,1,1)
end

return levels