-- Creator: Ralob
-- Last Modification: January 11, 2024
-- Version: alpha 0.2
-- Note: Many of the comments were translated by ChatGPT, so I apologize for any errors :]

debug_on = 1

love.graphics.setDefaultFilter("nearest", "nearest")

-- Resolution setup
local rs = require "libs/resolution_solution"
rs.conf({game_width = 800, game_height = 600, scale_mode = 1})
rs.setMode(rs.game_width, rs.game_height, {fullscreen = true, resizable = true})

-- Camera 
local camera = require "camera"
camera.set_zoom(1) -- Default
local camera_canvas = love.graphics.newCanvas(rs.game_width, rs.game_height)

-- Assets loader (Created by me :>)
local Aloader = require "libs/Aloader"
assets = {}

gamestates = require "gamestates/gamestates"

function love.load()
  love.physics.setMeter(32)
  World = love.physics.newWorld(0,700,true)
  World:setCallbacks(beginContact, endContact, preSolve, postSolve)
  
  -- Configures the asset loader.
  Aloader.conf(loading_draw,0.5)
  
  -- Loads the assets to be used.
  -- Note: The function provided as a parameter to `load_assets()` is called at the end of the loading process, functioning similar to `love.load`.
  local assets_data = require "assets/data/test_room"
  Aloader.load_assets(assets_data, assets,
  function()
    gamestates.set_state("playground")
  end
  )
end 

function love.update(dt)
  Aloader.update(dt)
  if Aloader.loaded then
    assets = Aloader.return_assets()
  end
  
  
  if not Aloader.loading then
    World:update(dt)
    gamestates.state_event("update", dt)
  end
end 

function love.draw()
  -- Camera Canvas
  camera.set()
    love.graphics.setCanvas(camera_canvas)
    love.graphics.clear(0.2,0.2,0.2,1)
   
    Aloader.draw()
   
    if not Aloader.loading then
      gamestates.state_event("draw")
    end
    
    love.graphics.setCanvas()
  camera.unset()
  
  -- Scale the game resolution.
  rs.push()
    love.graphics.draw(camera_canvas)
    
    -- Graphics that do not follow the camera logic
    love.graphics.scale(1.0000001)
    gamestates.state_event("debug")
  rs.pop()
    
  -- Debug that doesn't follow the simulated resolution.
  if debug_on > 0 then
    debug()
  end
end 

function love.resize(w, h)
  rs.resize()
end

-- Gamepad Callbacks
function love.joystickpressed(joy,button)
  gamestates.state_event("joystickpressed", joy, button)

  if button == 7 then
    debug_on = -debug_on
  end
end 

-- Keyboard Callbacks
function love.keypressed(key)
  gamestates.state_event("keypressed", key)

  if key == "f1" then
    debug_on = -debug_on
  end
end

-- Physics Callbacks
function beginContact(a, b, coll)
  gamestates.state_event("beginContact", a, b, coll)
end

function endContact(a,b,coll)
  gamestates.state_event("endContact", a, b, coll)
end 

function preSolve(a,b,coll)
  gamestates.state_event("preSolve", a, b, coll)
end

function postSolve(a,b,coll,c,d)
  gamestates.state_event("preSolve", a, b, coll, c, d)
end

-- Loading screen.
function loading_draw()
  love.graphics.setColor(0.2,0.5,0.3,1)
  love.graphics.rectangle("fill",0,0,rs.game_width,rs.game_height)

  love.graphics.setColor(1,1,1,1)
  love.graphics.print("IS LOADING", 5, 5, 0, 2,2)

  love.graphics.setColor(1,1,1,1)
  love.graphics.print("Temporary screen", 5, 50, 0, 1.5, 1.5)
end

function debug()
  love.graphics.setColor(1,1,1,1)
  love.graphics.print(table.concat({
    "World bodies:"..World:getBodyCount(),
    },"\n"),0,0,nil,0.8,0.8)
  Aloader.debug({1,1,0,1},0,20,0,0.6,0.6)
  gamestates.debug(0,50,0,0.6, 0.6)
  camera.debug(0,70,0,0.7,0.7)
  
  love.graphics.setColor(1,1,1,1)
  love.graphics.print("Alpha 0.2 \nJanuary 11, 2024", 0, 140, 0, 0.5,0.5)
end
